package pps.core.base.service;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.text.CharSequenceUtil;
import cn.hutool.core.util.ObjectUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.github.pagehelper.PageInfo;
import pps.core.base.entity.BaseLineAttenuationRateEnt;
import pps.core.base.entity.BaseProjectInfoEnt;
import pps.core.base.entity.BaseProjectInfoPredictedDataEnt;
import pps.core.base.mapper.BaseLineAttenuationRateMapper;
import pps.core.base.mapper.BaseProjectInfoMapper;
import pps.core.base.mapper.BaseProjectInfoPredictedDataMapper;
import pps.core.base.service.data.base_line_attenuation_rate.CreateBaseLineAttenuationRateInput;
import pps.core.base.service.data.base_project_info.*;
import xstartup.annotation.XService;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.data.XListResult;
import xstartup.data.XServiceResult;
import xstartup.data.XSingleResult;
import xstartup.error.XError;
import xstartup.feature.api.annotation.XApiGet;
import xstartup.feature.api.annotation.XApiPost;

import java.util.List;

@XService
public class BaseProjectInfoService {

    @XApiPost
    public XServiceResult createProjectInfo(XContext context, CreateBaseProjectInfoInput input) {
        return XServiceResult.OK;
    }

    @XApiPost
    public XServiceResult updateProjectInfo(XContext context, UpdateBaseProjectInfoInput input) {
        return XServiceResult.OK;
    }

    @XApiPost
    public XServiceResult deleteProjectInfo(XContext context, DeleteProjectInfoInput input) {
        BaseProjectInfoMapper mapper = context.getBean(BaseProjectInfoMapper.class);
        BaseProjectInfoEnt entity = mapper.selectOne(new LambdaQueryWrapper<BaseProjectInfoEnt>()
                .eq(BaseProjectInfoEnt::getId, input.getId()));
        if (ObjectUtil.isNull(entity)) {
            return XServiceResult.error(context, XError.NotFound);
        }
        entity.setIsDeleted(1);
        mapper.updateById(entity);
        return XServiceResult.OK;
    }

    @XApiGet
    public XSingleResult<GetBaseProjectInfoOutput> getProjectInfo(XContext context, GetProjectInfoInput input) {
        BaseProjectInfoMapper mapper = context.getBean(BaseProjectInfoMapper.class);
        BaseProjectInfoEnt entity = mapper.selectOne(new LambdaQueryWrapper<BaseProjectInfoEnt>()
                .eq(BaseProjectInfoEnt::getId, input.getId()));
        if (ObjectUtil.isNull(entity)) {
            return XSingleResult.error(context, XError.NotFound);
        }
        GetBaseProjectInfoOutput output = new GetBaseProjectInfoOutput();
        XCopyUtils.copyObject(entity, output);
        if (CharSequenceUtil.isNotBlank(entity.getProvince())) {
            output.setProvince(Integer.valueOf(entity.getProvince()));
        }
        BaseLineAttenuationRateMapper rateMapper = context.getBean(BaseLineAttenuationRateMapper.class);
        List<BaseLineAttenuationRateEnt> list = rateMapper.selectList(new LambdaQueryWrapper<BaseLineAttenuationRateEnt>()
                .eq(BaseLineAttenuationRateEnt::getLineId, entity.getId())
                .eq(BaseLineAttenuationRateEnt::getType, 1));
        if (CollUtil.isNotEmpty(list)) {
            List<CreateBaseLineAttenuationRateInput> rateInputs = XCopyUtils.copyNewList(list, CreateBaseLineAttenuationRateInput.class);
            output.setList(rateInputs);
        }
        return XSingleResult.success(output);
    }

    @XApiGet
    public XListResult<GetProjectInfoPredictedDataOutput> queryProjectInfoPredictedDataList(XContext context, GetProjectInfoPredictedDataInput input) {
        BaseProjectInfoPredictedDataMapper mapper = context.getBean(BaseProjectInfoPredictedDataMapper.class);
        List<BaseProjectInfoPredictedDataEnt> list = mapper.selectList(new LambdaQueryWrapper<BaseProjectInfoPredictedDataEnt>()
                .eq(ObjectUtil.isNotNull(input.getProjectId()), BaseProjectInfoPredictedDataEnt::getProjectId, input.getProjectId())
        );
        PageInfo<BaseProjectInfoPredictedDataEnt> pageInfo = new PageInfo<>(list);
        List<GetProjectInfoPredictedDataOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), GetProjectInfoPredictedDataOutput.class);
        return XListResult.success(outputs);
    }
}